<?php

namespace Drupal\worldmap;

use Drupal\Core\File\FileUrlGenerator;
use Drupal\Core\Extension\ModuleHandler;
use Drupal\Core\Extension\ExtensionList;
use Symfony\Component\Yaml\Yaml as YamlParser;

/**
 * Helper service for World map block.
 */
final class MapHelper {

  /**
   * Path to country data.
   */
  const COUNTRY_YAML = __DIR__ . '/map_data.yml';

  /**
   * Module handler core service.
   *
   * @var \Drupal\Core\Extension\ModuleHandler
   */
  private ModuleHandler $moduleHandler;

  /**
   * File URL generator core service.
   *
   * @var \Drupal\Core\File\FileUrlGenerator
   */
  private FileUrlGenerator $fileUrlGenerator;

  /**
   * Extension list core service.
   *
   * @var \Drupal\Core\Extension\ExtensionList
   */
  private ExtensionList $extensionList;

  /**
   * Class constructor.
   *
   * @param \Drupal\Core\File\FileUrlGenerator $file_url_generator
   *   Instance of file_url_generator core service.
   * @param \Drupal\Core\Extension\ModuleHandler $module_handler
   *   Instance of module_handler core service.
   * @param \Drupal\Core\Extension\ExtensionList $extension_list
   *   Instance of extension.list.module core service.
   */
  public function __construct(FileUrlGenerator $file_url_generator, ModuleHandler $module_handler, ExtensionList $extension_list) {
    $this->fileUrlGenerator = $file_url_generator;
    $this->moduleHandler = $module_handler;
    $this->extensionList = $extension_list;
  }

  /**
   * Get country options.
   *
   * @return array
   *   Prepared options array.
   */
  public function getCountryOptions(): array {
    $options = [];
    $country_list = [];
    $this->moduleHandler->invokeAll('worldmap_countries', [&$country_list]);
    foreach ($country_list as $code => $country) {
      $options[$code] = $country;
    }
    uasort($options, fn ($a, $b) => strcmp($a, $b));

    return $options;
  }

  /**
   * Get map data.
   *
   * @return array
   *   Countries data array.
   */
  public function getMapData(array $block_config): array {
    $data = YamlParser::parseFile(self::COUNTRY_YAML);
    $country_list = [];

    $this->moduleHandler->invokeAll('worldmap_countries', [&$country_list]);

    foreach ($data as $code => &$country) {
      $country_name_tmp = $country['name'];
      $country['name'] = !empty($country_list[$code]) ? $country_list[$code] : $country_name_tmp;
      $country['selected'] = FALSE;

      if (!empty($block_config['countries'][$code])) {
        $country['selected'] = TRUE;
      }

      $links = $block_config['links'];

      if (!empty($links[$code])) {
        $country['link'] = $links[$code];
      }
    }

    uasort($data, function ($a, $b) {
      return strcmp($a['name'], $b['name']);
    });

    return $data;
  }

  /**
   * Get the module's install directory.
   *
   * @return string
   *   Worldmap module directory path.
   */
  public function getModuleInstalledPath(): string {
    return $this->fileUrlGenerator->generateAbsoluteString(
      $this->extensionList->getPath('worldmap')
    );
  }

}
